'use strict';

var mandrill = require('mandrill-api/mandrill');
var errTree = require('err-tree');
var _ = require('lodash');
var Promise = require('bluebird');
var Datauri = require('datauri');
var defaultValue = require('./utils/templateHelper').defaultValue;
var dAsync = Promise.promisify(Datauri);
var debug = require('debug')('core:email');
var embeddedImages = {};

function setGoogleAction(content, call2Action, hrefCall2Action) {
    var call2ActionContent = call2Action && call2Action.content;
    var hrefCall2ActionContent = hrefCall2Action && hrefCall2Action.content;
    var googleAction = defaultValue('GMAIL_ACTION', content, '');
    if (googleAction.content && typeof googleAction.content !== 'object') {
        // If content is not an object or an empty string, assume it should be displayed as is
        return;
    }

    var call2ActionOptions = {};

    if (call2ActionContent && hrefCall2ActionContent) {
        call2ActionOptions = {
            potentialAction: {
                name: call2ActionContent,
                target: hrefCall2ActionContent
            }
        };
    }
    var content = _.merge({
        '@context': 'http://schema.org',
        '@type': 'EmailMessage',
        potentialAction: {
            '@type': 'ViewAction',
        }
    },
    call2ActionOptions,
    googleAction.content);

    // If we can deduce an email action, then generate one
    if (content.potentialAction && content.potentialAction.name && content.potentialAction.target) {
        googleAction.content = '<script type="application/ld+json">' +
            JSON.stringify(content) +
            '</script>';
    }
}

function Email(options) {
    this.options = _.defaults({}, options);
    if (!this.client && this.options.mandrillApiKey) {
        this.client = new mandrill.Mandrill(this.options.mandrillApiKey);
        this.key = this.options.mandrillApiKey;
    }
}

Email.prototype.EmailError = errTree(function EmailError(email, error) {
    errTree.BasicError.apply(this, [JSON.stringify({email: email, error: error})]);
});

/**
 * Creates a new e-mail message to be sent using a template
 * @param {[type]} templateName    [description]
 * @param {[type]} templateContent [description]
 * @param {[type]} config          [description]
 * @return {Promise} Resolve a message
 */
Email.prototype.getTemplateMessage = function(templateName, templateContent, config) {
    return new Promise(function(resolve) {
        var message = {};
        message.template_content = templateContent || [];
        if (_.startsWith(templateName, 'default')) {
            defaultValue('URL_BASE', message.template_content, 'https://up.coorpacademy.com');
            defaultValue(
                'IMG_HEADER', message.template_content,
                'https://static.coorpacademy.com/emails/default/img-header-v1-760x334.png'
            );
            defaultValue('TITLE', message.template_content, '');
            defaultValue('SUBTITLE', message.template_content, '');
            defaultValue('custom', message.template_content, '');
            defaultValue('HREF_LOGO', message.template_content, 'https://www.coorpacademy.com');
            defaultValue(
                'IMG_LOGO', message.template_content,
                'https://static.coorpacademy.com/emails/default/logo-v1-1024x208.png'
            );
            defaultValue('PRIMARY_COLOR', message.template_content, '#3498db');
            defaultValue('HREF_FACEBOOK', message.template_content, 'https://www.facebook.com/coorpacademy');
            defaultValue('HREF_GOOGLEPLUS', message.template_content,
                'https://plus.google.com/103035846529483174814/about');
            defaultValue('HREF_LINKEDIN', message.template_content, 'https://www.linkedin.com/company/coorpacademy');

            var call2Action = _.find(message.template_content, {name: 'CALL2ACTION'});
            if (call2Action === undefined) {
                defaultValue('call2action', message.template_content, '');
            }
            else {
                defaultValue('HREF_CALL2ACTION', message.template_content, '');
            }

            var hrefCall2Action =_.find(message.template_content, {name: 'HREF_CALL2ACTION'});
            setGoogleAction(message.template_content, call2Action, hrefCall2Action);

            if (config.hideSocialNetworkLinks) {
                message.template_content.push({
                    name: 'social_display', content: ''
                });
            }
            if (config.hideHeaderImage) {
                message.template_content.push({
                    name: 'img_header_display', content: ''
                });
            }
        }
        message.async = true;
        message.message = {};
        message.template_name = templateName;
        message.message.subject = config.subject;
        message.message.to = [{
            type: 'to',
            name: config.recipient.name,
            email: config.recipient.email
        }];
        message.message.from_email = config.sender.email;
        message.message.from_name = config.sender.name;
        // use same for other merge template engine
        message.message.global_merge_vars = message.template_content;
        if (config.images) {
            var promisesImages = [];
            config.images.forEach(function(image) {
                promisesImages.push(this.embedImage(image.imageName, image.imagePath));
            }.bind(this));
            return Promise.all(promisesImages).
                then(function(images) {
                    message.message.images = images;
                    debug('getTemplateMessage', message);
                    return resolve(message);
                })
            ;
        }
        else {
            debug('getTemplateMessage', message);
            return resolve(message);
        }
    }.bind(this));
};

/**
 * DEPRECATED
 * @type {[type]}
 */
Email.prototype.SendTemplate = Email.prototype.getTemplateMessage;

Email.prototype._onSuccessCallback = function(result, callback) {
    debug('successCallback', result);
    var status = !result || !result.length ? null : result[0].status;
    if (status !== 'sent' && status !== 'queued') {
        return callback(
            {
                status: 'error',
                code: -1,
                name: 'MandrillResultError',
                message: 'Mandrill reports a bad status for e-mail sent: ' + status
            }
        );
    }
    return callback(null, result);
};

/**
 * DEPRECATED
 * @param  {[type]}   sendTemplate [description]
 * @param  {Function} callback     [description]
 * @return {[type]}                [description]
 */
Email.prototype.send = function(sendTemplate, callback) {
    this.client.messages.sendTemplate(
        sendTemplate,
        function(result) {
            return this._onSuccessCallback(result, callback);
        }.bind(this),
        callback
    );
};

Email.prototype.sendAsync = Promise.promisify(Email.prototype.send);

Email.prototype.embedImage = function(imageName, imagePath) {
    return new Promise(function(resolve) {
        if (embeddedImages[imageName]) {
            return resolve(embeddedImages[imageName]);
        }
        return dAsync(imagePath).then(function(content) {
            debug('embedImage', content);
            /*
             * mandrill api image embedding
             *
             * images {Array} an array of embedded images to add to the message
             * - images[] {Object} a single embedded image
             * - type {String} the MIME type of the image - must start with "image/"
             * - name {String} the Content ID of the image - use <img src="cid:THIS_VALUE">
             *   to reference the image in your HTML content
             * - content {String} the content of the image as a base64-encoded string
             */
            embeddedImages[imageName] = {
                type: content[1].mimetype,
                name: imageName,
                content: content[1].base64
            };
            resolve(embeddedImages[imageName]);
        });
    });
};

/**
 * get the content of a messahe
 * @param  {[type]} id [description]
 * @return {[type]}    [description]
 */
Email.prototype.content = function(id) {
    return new Promise(function(resolve, reject) {
        this.client.messages.content({
            key: this.key,
            id: id
        }, resolve, reject);
    }.bind(this));
};

module.exports = Email;
