'use strict';

var _ = require('lodash');
var sinon = require('sinon');
var has = sinon.match.has;
var any = sinon.match.any;
var defined = sinon.match.defined;
var chai = require('chai');
var sinonChai = require('sinon-chai');
chai.use(sinonChai);
var expect = chai.expect;
var mandrill = require('mandrill-api/mandrill');
var Email = require('../../lib/email/email');
var path = require('path');
var fs = require('fs');
var _ = require('lodash');

describe('email', function() {
    describe('send', function() {
        var messagesMock;

        beforeEach(function() {
            messagesMock = {sendTemplate: sinon.spy()};
            sinon.stub(mandrill, 'Mandrill').returns({messages: messagesMock});
        });
        afterEach(function() {
            mandrill.Mandrill.restore();
        });

        it('should send an e-mail with mandrill using template', function() {
            var emailHandler = new Email({mandrillApiKey: 'mandrillApiKey'});
            var sendTemplate = 'the template to be sent';

            emailHandler.send(sendTemplate);

            expect(messagesMock.sendTemplate).to.have.been.calledWith(sendTemplate);
        });
    });

    describe('callbacks', function() {
        var callback = sinon.spy();

        beforeEach(function() {
            callback.reset();
        });

        describe('when mandrill is on success', function() {
            function mandrillResultWithStatus(status) {
                return [{status: status}];
            }
            function whenMandrillIsOnSuccess(result) {
                var mandrillStubbed = new mandrill.Mandrill('mandrillApiKey');

                beforeEach(function() {
                    sinon.stub(mandrillStubbed.messages, 'sendTemplate', function(email, onSuccess) {
                        return onSuccess(result);
                    });
                    sinon.stub(mandrill, 'Mandrill').returns(mandrillStubbed);
                });
                afterEach(function() {
                    mandrillStubbed.messages.sendTemplate.restore();
                    mandrill.Mandrill.restore();
                });
            }
            function itShouldReturnSuccess(result) {
                it('should return success', function() {
                    var emailHandler = new Email({mandrillApiKey: 'mandrillApiKey'});

                    emailHandler.send('template', callback);

                    expect(callback).to.have.been.calledWith(null, result);
                });
            }
            function itShouldReturnError(mandrillStatus) {
                it('should return error', function() {
                    var emailHandler = new Email({mandrillApiKey: 'mandrillApiKey'});

                    emailHandler.send('template', callback);

                    expect(callback).to.have.been.calledWithMatch(has('code', -1));
                    expect(callback).to.have.been.calledWithMatch(
                        has('message', 'Mandrill reports a bad status for e-mail sent: ' + mandrillStatus)
                    );
                    expect(callback).to.have.been.calledWithMatch(has('name', 'MandrillResultError'));
                    expect(callback).to.have.been.calledWithMatch(has('status', 'error'));
                    expect(callback).to.not.have.been.calledWithMatch(any, defined);
                });
            }
            describe('with status sent', function() {
                var sent = mandrillResultWithStatus('sent');
                whenMandrillIsOnSuccess(sent);
                itShouldReturnSuccess(sent);
            });

            describe('with status queued', function() {
                var queued = mandrillResultWithStatus('queued');
                whenMandrillIsOnSuccess(queued);
                itShouldReturnSuccess(queued);
            });

            describe('with another status', function() {
                var weird = mandrillResultWithStatus('weird_status');
                whenMandrillIsOnSuccess(weird);
                itShouldReturnError('weird_status');
            });

            describe('with no result', function() {
                whenMandrillIsOnSuccess(null);
                itShouldReturnError('null');
            });

            describe('getting content', function() {
                var mandrillStubbed = new mandrill.Mandrill('mandrillApiKey');
                beforeEach(function() {
                    sinon.stub(mandrillStubbed.messages, 'content', function(params, onsuccess) {
                        return onsuccess(params);
                    });
                    sinon.stub(mandrill, 'Mandrill').returns(mandrillStubbed);
                });

                afterEach(function() {
                    mandrillStubbed.messages.content.restore();
                    mandrill.Mandrill.restore();
                });
                it('should return success', function(done) {
                    var emailHandler = new Email({mandrillApiKey: 'mandrillApiKey'});

                    emailHandler.content('id')
                        .then(function(result) {
                            expect(result).to.have.property('id', 'id');
                            expect(result).to.have.property('key', 'mandrillApiKey');
                            done();
                        })
                        .catch(done)
                    ;
                });
            });
        });

        describe('when mandrill is on error', function() {
            var mandrillStubbed = new mandrill.Mandrill('mandrillApiKey');
            var mandrillError = 'error';

            beforeEach(function() {
                sinon.stub(mandrillStubbed.messages, 'sendTemplate', function(email, onSuccess, onError) {
                    return onError(mandrillError);
                });
                sinon.stub(mandrill, 'Mandrill').returns(mandrillStubbed);
            });
            afterEach(function() {
                mandrillStubbed.messages.sendTemplate.restore();
                mandrill.Mandrill.restore();
            });

            it('should return error', function() {
                var emailHandler = new Email({mandrillApiKey: 'mandrillApiKey'});

                emailHandler.send('template', callback);

                expect(callback).to.have.been.calledWith(mandrillError);
            });
        });
    });

    describe('constructors', function() {
        var emailHandler = new Email({mandrillApiKey: 'mandrillApiKey'});
        var img_logo = path.resolve(__dirname, '../fixtures/img_logo.png');
        var img_logo_b64_path = path.resolve(__dirname, '../fixtures/img_logo.b64');
        var img_logo_b64;
        before(function(done) {
            fs.readFile(img_logo_b64_path, 'utf8', function(err, data) {
              if (err) {
                return done(err);
              }
              img_logo_b64 = data;
              done();
            });
        });
        it('should handle prepare email', function(done) {
            emailHandler.getTemplateMessage('default', [{
                name: 'TITLE',
                content: 'some hard-coded title'
            }], {
                subject: '',
                recipient: {
                    name: '',
                    email: ''
                },
                sender: {
                    name: '',
                    email: ''
                }
            }).then(function(email) {
                expect(email.template_name).to.be.equal('default');
            }).nodeify(done);
        });
        it('should handle accept empty values', function(done) {
            emailHandler.getTemplateMessage('default', undefined, {
                subject: '',
                recipient: {
                    name: '',
                    email: ''
                },
                sender: {
                    name: '',
                    email: ''
                }
            }).then(function(email) {
                expect(email.template_name).to.be.equal('default');
            }).nodeify(done);
        });

        it('should handle transform embedded images', function(done) {
            emailHandler.getTemplateMessage('default', undefined, {
                subject: '',
                recipient: {
                    name: '',
                    email: ''
                },
                sender: {
                    name: '',
                    email: ''
                },
                images: [{
                    imageName: 'img_logo',
                    imagePath: img_logo
                    }
                ]
            }).then(function(email) {
                expect(email.message.images).to.be.an('array');
                expect(email.message.images).to.have.length(1);
                expect(email.message.images[0]).to.have.property('content', img_logo_b64);
                expect(email.message.images[0]).to.have.property('name', 'img_logo');
                expect(email.message.images[0]).to.have.property('type', 'image/png');
            }).nodeify(done);
        });

        it('should not need imagePath for already processed images', function(done) {
            emailHandler.getTemplateMessage('default', undefined, {
                subject: '',
                recipient: {
                    name: '',
                    email: ''
                },
                sender: {
                    name: '',
                    email: ''
                },
                images: [{
                    imageName: 'img_logo'
                    }
                ]
            }).then(function(email) {
                expect(email.message.images).to.be.an('array');
                expect(email.message.images).to.have.length(1);
                expect(email.message.images[0]).to.have.property('content', img_logo_b64);
                expect(email.message.images[0]).to.have.property('name', 'img_logo');
                expect(email.message.images[0]).to.have.property('type', 'image/png');
            }).nodeify(done);
        });
        it('should hide header', function(done) {
            emailHandler.getTemplateMessage('default', undefined, {
                subject: '',
                recipient: {
                    name: '',
                    email: ''
                },
                sender: {
                    name: '',
                    email: ''
                },
                hideHeaderImage: true
            }).then(function(email) {
                expect(_.findIndex(email.template_content, {name: 'img_header_display', content: ''})).to.not.equal(-1);
            }).nodeify(done);
        });
        it('should show header', function(done) {
            emailHandler.getTemplateMessage('default', undefined, {
                subject: '',
                recipient: {
                    name: '',
                    email: ''
                },
                sender: {
                    name: '',
                    email: ''
                },
                hideHeaderImage: false
            }).then(function(email) {
                expect(_.findIndex(email.template_content, {name: 'img_header_display', content: ''})).to.be.equal(-1);
            }).nodeify(done);
        });
        it('should hide social', function(done) {
            emailHandler.getTemplateMessage('default', undefined, {
                subject: '',
                recipient: {
                    name: '',
                    email: ''
                },
                sender: {
                    name: '',
                    email: ''
                },
                hideSocialNetworkLinks: true
            }).then(function(email) {
                expect(_.findIndex(email.template_content, {name: 'social_display', content: ''})).to.not.equal(-1);
            }).nodeify(done);
        });
        it('should show social', function(done) {
            emailHandler.getTemplateMessage('default', undefined, {
                subject: '',
                recipient: {
                    name: '',
                    email: ''
                },
                sender: {
                    name: '',
                    email: ''
                },
                hideSocialNetworkLinks: false
            }).then(function(email) {
                expect(_.findIndex(email.template_content, {name: 'social_display', content: ''})).to.be.equal(-1);
            }).nodeify(done);
        });

        it('should have EmailError', function() {
            var emailError = new emailHandler.EmailError();
            expect(emailError).to.have.property('name', 'EmailError');
        });
    });

    describe('getTemplateMessage', function () {
        var email;
        var config;
        var content;

        beforeEach(function() {
            email = new Email({mandrillApiKey: 'mandrillApiKey'});
            config = {
                subject: 'some-subject',
                sender: {
                    email: 'some-sender-email@thing.com',
                    name: 'some-sender-name'
                },
                recipient: {
                    email: 'some-recipient-email@thing.com',
                    name: 'some-recipient-name'
                }
            };
            content = [];
        });

        it('should return a Promise', function() {
            expect(email.getTemplateMessage('some-template', undefined, config).then).to.be.a('function');
        });

        it('should set values in email', function(done) {
            email.getTemplateMessage('some-template', undefined, config)
            .then(function(message) {
                expect(message.template_name).to.equal('some-template');
                expect(message.async).to.be.true;
                done();
            })
            .catch(done);
        });

        it('should integrate values in config', function(done) {
            email.getTemplateMessage('some-template', content, config)
            .then(function(message) {
                expect(message.message.subject).to.equal('some-subject');
                expect(message.message.to).to.be.an('Array').of.length(1);
                expect(message.message.to[0].type).to.equal('to');
                expect(message.message.to[0].email).to.equal('some-recipient-email@thing.com');
                expect(message.message.to[0].name).to.equal('some-recipient-name');
                expect(message.message.from_email).to.equal('some-sender-email@thing.com');
                expect(message.message.from_name).to.equal('some-sender-name');
                done();
            })
            .catch(done);
        });

        it('should use an empty array for template_content if content is not defined', function(done) {
            email.getTemplateMessage('some-template', undefined, config)
            .then(function(message) {
                expect(message.template_content).to.be.an('Array').of.length(0);
                expect(message.message.global_merge_vars).to.equal(message.template_content);
                done();
            })
            .catch(done);
        });

        describe('with default template', function() {
            var scriptTagStart = '<script type="application/ld+json">';
            var scriptTagEnd = '</script>';

            function findContentFor(message, name) {
                return _.find(message.template_content, {name: name}).content;
            }

            function extractScriptOptions(content) {
                var scriptJSON = content
                    .replace(scriptTagStart, '')
                    .replace(scriptTagEnd, '');
                return JSON.parse(scriptJSON);
            }

            it('should give default values to content', function(done) {
                email.getTemplateMessage('default', [{
                    name: 'BLABLA',
                    content: 'some-content'
                }], config)
                .then(function(message) {
                    expect(findContentFor(message, 'URL_BASE')).to.equal('https://up.coorpacademy.com');
                    expect(findContentFor(message, 'TITLE')).to.equal('');
                    expect(findContentFor(message, 'SUBTITLE')).to.equal('');
                    expect(findContentFor(message, 'custom')).to.equal('');
                    expect(findContentFor(message, 'GMAIL_ACTION')).to.equal('');

                    // expect(findContentFor(message, 'CALL2ACTION')).to.equal('');
                    expect(_.find(message.template_content, {name: 'CALL2ACTION'})).to.equal(undefined);
                    expect(_.find(message.template_content, {name: 'HREF_CALL2ACTION'})).to.equal(undefined);
                    done();
                })
                .catch(done);
            });

            it('should not have default values override passed content', function(done) {
                email.getTemplateMessage('default', [{
                    name: 'SOME-NAME',
                    content: 'some-content'
                }, {
                    name: 'URL_BASE',
                    content: 'some-url'
                }], config)
                .then(function(message) {
                    expect(findContentFor(message, 'SOME-NAME')).to.equal('some-content');
                    expect(findContentFor(message, 'URL_BASE')).to.equal('some-url');
                    done();
                })
                .catch(done);
            });

            it('should create a default image header', function(done) {
                email.getTemplateMessage('default', [], config)
                .then(function(message) {
                    var filteredContent = message.template_content.filter(function(value) {
                        return value.name === 'IMG_HEADER';
                    })
                    expect(filteredContent).to.have.length(1);
                    expect(filteredContent[0]).to.have.property('name', 'IMG_HEADER');
                    expect(filteredContent[0].content).to.match(/^http/); // should be link
                    expect(filteredContent[0].content).to.match(/\.(png|jpe?g|gif)$/); // should be an image
                    done();
                })
                .catch(done);
            });

            it('should have GMAIL_ACTION be an empty string if it and CALL2ACTION are not defined', function(done) {
                email.getTemplateMessage('default', [], config)
                .then(function(message) {
                    expect(findContentFor(message, 'GMAIL_ACTION')).to.equal('');
                    done();
                })
                .catch(done);
            });

            it('should have GMAIL_ACTION be an empty string if it and HREF_CALL2ACTION are not defined', function(done) {
                email.getTemplateMessage('default', [], config)
                .then(function(message) {
                    expect(findContentFor(message, 'GMAIL_ACTION')).to.equal('');
                    done();
                })
                .catch(done);
            });

            it('should keep GMAIL_ACTION\'s content if it is a non-empty string', function(done) {
                email.getTemplateMessage('default', [{
                    name: 'GMAIL_ACTION',
                    content: 'google'
                }, {
                    name: 'HREF_CALL2ACTION',
                    content: 'href'
                }, {
                    name: 'CALL2ACTION',
                    content: 'call'
                }], config)
                .then(function(message) {
                    expect(findContentFor(message, 'GMAIL_ACTION')).to.equal('google');
                    done();
                })
                .catch(done);
            });

            it('should scriptify GMAIL_ACTION\'s content if it is an object', function(done) {
                var expectedContent = {
                    potentialAction: {
                        name: 'some-name',
                        target: 'some-target'
                    }
                };
                email.getTemplateMessage('default', [{
                    name: 'GMAIL_ACTION',
                    content: expectedContent
                }], config)
                .then(function(message) {
                    var content = findContentFor(message, 'GMAIL_ACTION');
                    expect(content).to.be.a('string');
                    expect(content.indexOf(scriptTagStart)).to.equal(0);
                    expect(content.indexOf(scriptTagEnd)).to.equal(content.length - scriptTagEnd.length);
                    expect(extractScriptOptions(content)).to.exist;
                    done();
                })
                .catch(done);
            });

            it('should scriptify GMAIL_ACTION\'s content if it an empty string', function(done) {
                email.getTemplateMessage('default', [{
                    name: 'GMAIL_ACTION',
                    content: ''
                }, {
                    name: 'HREF_CALL2ACTION',
                    content: 'href'
                }, {
                    name: 'CALL2ACTION',
                    content: 'call'
                }], config)
                .then(function(message) {
                    var content = findContentFor(message, 'GMAIL_ACTION');
                    expect(content).to.be.a('string');
                    expect(content.indexOf(scriptTagStart)).to.equal(0);
                    expect(content.indexOf(scriptTagEnd)).to.equal(content.length - scriptTagEnd.length);
                    expect(extractScriptOptions(content)).to.exist;
                    done();
                })
                .catch(done);
            });

            it('should merge default and custom GMAIL_ACTION content values if it is an object', function(done) {
                email.getTemplateMessage('default', [{
                    name: 'GMAIL_ACTION',
                    content: {
                        '@type': 'some-type',
                        'some-unknown-field': 'some-unknown-value',
                        potentialAction: {
                            '@type': 'some-action-type',
                            name: 'some-name',
                            target: 'some-target'
                        }
                    }
                }], config)
                .then(function(message) {
                    var content = findContentFor(message, 'GMAIL_ACTION');
                    var options = extractScriptOptions(content);
                    expect(options).to.have.property('@context', 'http://schema.org');
                    expect(options).to.have.property('@type', 'some-type');
                    expect(options).to.have.property('some-unknown-field', 'some-unknown-value');
                    expect(options).to.have.property('potentialAction');
                    expect(options.potentialAction).to.have.property('@type', 'some-action-type');
                    expect(options.potentialAction).to.have.property('name', 'some-name');
                    expect(options.potentialAction).to.have.property('target', 'some-target');
                    done();
                })
                .catch(done);
            });

            it('should have GMAIL_ACTION be a simple script if it is not defined but there is a C2A', function(done) {
                email.getTemplateMessage('default', [{
                    name: 'HREF_CALL2ACTION',
                    content: 'href'
                }, {
                    name: 'CALL2ACTION',
                    content: 'call'
                }], config)
                .then(function(message) {
                    var content = findContentFor(message, 'GMAIL_ACTION');
                    var options = extractScriptOptions(content);
                    expect(options).to.have.property('@type', 'EmailMessage');
                    expect(options).to.have.property('potentialAction');
                    expect(options.potentialAction).to.have.property('@type', 'ViewAction');
                    expect(options.potentialAction).to.have.property('name', 'call');
                    expect(options.potentialAction).to.have.property('target', 'href');
                    done();
                })
                .catch(done);
            });
        });
    });
});
