'use strict';

var expect = require('chai').expect;
var errorHandler = require('../../lib/errorHandler');

describe('errorHandler', function() {
    describe('safeEnv', function() {
        var oldEnv;

        beforeEach(function() {
            oldEnv = process.env.NODE_ENV;
        });

        it('should detect development env', function() {
            process.env.NODE_ENV = 'development';
            var safeEnv = errorHandler.safeEnv();
            expect(safeEnv).to.be.true;
        });

        it('should detect test env', function() {
            process.env.NODE_ENV = 'test';
            var safeEnv = errorHandler.safeEnv();
            expect(safeEnv).to.be.true;
        });

        it('should detect staging env', function() {
            process.env.NODE_ENV = 'staging';
            var safeEnv = errorHandler.safeEnv();
            expect(safeEnv).to.be.false;
        });

        it('should detect production env', function() {
            process.env.NODE_ENV = 'production';
            var safeEnv = errorHandler.safeEnv();
            expect(safeEnv).to.be.false;
        });

        afterEach(function() {
            if (oldEnv) {
                process.env.NODE_ENV = oldEnv;
            }
        });
    });
    it('should generate a unique id', function() {
        var Err1 = errorHandler.generateId();
        var Err2 = errorHandler.generateId();
        expect(Err1).to.not.equal(Err2);
    });

    it('should generate a unique id with custom suffix', function() {
        var Err1 = errorHandler.generateId('CUSTOM');
        expect(Err1).to.contain('CUSTOM');
    });

    it('should return wrapped object', function() {
        var wrapped = errorHandler.wrap({});
        expect(wrapped).to.have.property('id');
        expect(wrapped.id).to.contain('ERR');
    });

    it('should return none wrapped object', function() {
        var wrapped = errorHandler.wrap({id: 'exist'});
        expect(wrapped).to.have.property('id');
        expect(wrapped.id).to.be.equal('exist');
    });

    it('should cleanError with stack in development', function() {
        var error = new Error('boom');
        var oldEnv = process.env.NODE_ENV;
        process.env.NODE_ENV = 'development';
        var cleanedError = errorHandler.cleanifyError(error);
        expect(cleanedError).to.have.property('stack');
        process.env.NODE_ENV = oldEnv;
    });

    it('should get express handler', function() {
        var middleware = errorHandler.expressErrorMiddleware();
        expect(middleware).to.be.an('function');
        expect(middleware.length).to.be.equal(4);
    });

    it('should have CoorpError', function() {
        var error = new errorHandler.CoorpError();
        expect(error).to.have.property('id');
    });

    describe('middleware', function() {
        var middleware = errorHandler.expressErrorMiddleware({
            errorView: 'errorView'
        });
        var req;
        var res;
        var oldEnv;

        beforeEach(function() {
            errorHandler.logger = {
                error: function() {
                }
            };
            req = {
                xhr: false
            };
            res = {
                status: function(status) {
                    this._status = status;
                    return this;
                },
                render: function(view, model) {
                    this._view = view;
                    this._render = true;
                    this._model = model;
                    return this;
                },
                json: function(model) {
                    this._model = model;
                    this._json = true;
                    return this;
                }
            };
            oldEnv = process.env.NODE_ENV;
        });

        it('should send json on ajax request with stack', function() {
            process.env.NODE_ENV = 'development';
            var error = new Error('boom');
            req.xhr = true;
            middleware(error, req, res);
            expect(res._status).to.be.equal(500);
            expect(res._model).to.have.property('success');
            expect(res._model).to.have.property('stack');
            expect(res._json).to.be.true;
        });

        it('should send json on ajax request without stack in production', function() {
            process.env.NODE_ENV = 'production';
            var error = new Error('boom');
            req.xhr = true;
            middleware(error, req, res);
            expect(res._status).to.be.equal(500);
            expect(res._model).to.have.property('success');
            expect(res._model).to.not.have.property('stack');
            expect(res._json).to.be.true;
        });

        it('should send render on html request', function() {
            var error = new Error('boom');
            req.xhr = false;
            middleware(error, req, res);
            expect(res._status).to.be.equal(500);
            expect(res._view).to.be.equal('errorView');
            expect(res._model).to.have.property('error');
            expect(res._render).to.be.true;
        });

        afterEach(function() {
            errorHandler.logger = console;
            if (oldEnv) {
                process.env.NODE_ENV = oldEnv;
            }
        });
    });
});
