'use strict';

var sinon = require('sinon');
var expect = require('chai').expect;
var proxyquire = require('proxyquire').noCallThru();

var libPath = '../../lib/errorTracker';

describe('errorTracker', function() {
    var errorTracker, errorStub, notifySpy, registerSpy, config, version;

    beforeEach(function() {
        errorStub = sinon.stub(console, 'error');
        errorTracker = proxyquire(libPath, {
            bugsnag: {
                notify: notifySpy = sinon.spy(),
                register: registerSpy = sinon.spy()
            }
        });
        config = {
            server: {
                apiKey: 'some-key'
            }
        };
        version = '1.0.0';
    });

    afterEach(function() {
        // Remove listener added in the module to avoid error handler to be called multiple times
        process.removeAllListeners('unhandledRejection');
        console.error.restore();
    });

    it('should have the same methods as the bugsnag package', function() {
        expect(errorTracker).to.have.property('notify')
            .that.is.a('function');
    });

    it('should have a collect method', function() {
        expect(errorTracker).to.have.property('collect')
            .that.is.a('function');
    });

    it('should notify Bugsnag and console.error() on unhandledRejection events', function() {
        var expectedError = 'some message on unhandledRejection';
        errorTracker.collect(config, version);
        process.emit('unhandledRejection', expectedError);

        expect(notifySpy.callCount).to.equal(1);
        expect(notifySpy.getCall(0).args).to.deep.equal([expectedError]);

        expect(errorStub.callCount).to.equal(1);
        expect(errorStub.getCall(0).args[0])
            .to.match(/some message on unhandledRejection/) // Print the error
        .to.not.match(/at Context/); // no stack trace
    });

    it('should console.error() on unhandledRejection events with stacktrace if available', function() {
        var expectedError = new Error('some error on unhandledRejection');
        errorTracker.collect(config, version);
        process.emit('unhandledRejection', expectedError);

        expect(errorStub.callCount).to.equal(1);
        expect(errorStub.getCall(0).args[0])
            .to.match(/some error on unhandledRejection/) // Print the error
        .to.match(/at Context/); // Print the stack trace
    });

    it('should throw an error if config is missing', function() {
        expect(function() {
            errorTracker.collect();
        }).to.throw();

        expect(function() {
            errorTracker.collect(null, version);
        }).to.throw();
    });

    it('should throw an error if config.server.apiKey does not exist', function() {
        expect(function() {
            errorTracker.collect({}, version);
        }).to.throw(/missing errorTracker server.apiKey/);

        expect(function() {
            errorTracker.collect({
                server: {}
            }, version);
        }).to.throw(/missing errorTracker server.apiKey/);
    });

    it('should not re-init if collect() is called multiple times', function() {
        errorTracker.collect(config, version);
        config.server.apiKey = 'some-other-key';
        errorTracker.collect(config, version);
        expect(registerSpy.callCount).to.equal(1);
        expect(registerSpy.getCall(0).args[0]).to.equal('some-key');
    });

    it('should be able to call collect() if first call failed', function() {
        expect(function() {
            errorTracker.collect({}, version);
        }).to.throw(/missing errorTracker server.apiKey/);

        errorTracker.collect(config, version);
        expect(registerSpy.callCount).to.equal(1);
        expect(registerSpy.getCall(0).args[0]).to.equal('some-key');
    });
});
