'use strict';

var sinon = require('sinon');
var expect = require('chai').expect;
var errorTracker = require('../../lib/errorTracker');
var MemoryStream = require('memorystream');
var express = require('express');

var logger;
var consoleStub;

describe('logger', function() {
    before(function() {
        logger = require('../../lib/logger');
    });

    describe('log & info', function() {
        it('should use console.log as output when calling logger.info', function() {
            // Warning: Mocking console.log hides the output of mocha, but not when the test fails
            consoleStub = sinon.stub(console, 'log').returns();
            logger.log('some %s', 'error');
            expect(consoleStub.callCount).to.equal(1);
            expect(consoleStub.getCall(0).args).to.deep.equal(['some %s', 'error']);
            console.log.restore();
        });

        it('should use console.info as output when calling logger.info', function() {
            consoleStub = sinon.stub(console, 'info').returns();
            logger.info('some %s', 'error');
            expect(consoleStub.callCount).to.equal(1);
            expect(consoleStub.getCall(0).args).to.deep.equal(['some %s', 'error']);
            console.info.restore();
        });
    });

    describe('warn', function() {
        var errorTrackerNotifyStub;
        var expectedOptions;

        beforeEach(function() {
            consoleStub = sinon.stub(console, 'warn').returns();
            errorTrackerNotifyStub = sinon.stub(errorTracker, 'notify').returns();

            expectedOptions = {
                severity: 'warning'
            };
        });

        afterEach(function() {
            console.warn.restore();
            errorTracker.notify.restore();
        });

        it('should not do anything if no arguments were passed', function() {
            logger.warn();

            expect(consoleStub.callCount).to.equal(0);
            expect(errorTrackerNotifyStub.callCount).to.equal(0);
        });

        it('should simply pass arguments to console.warn', function() {
            logger.warn('some %s %s', 'warn', 2);

            expect(consoleStub.callCount).to.equal(1);
            expect(consoleStub.getCall(0).args).to.deep.equal(['some %s %s', 'warn', 2]);
        });

        it('should use utils.format to format the message when sending to Bugsnag', function() {
            logger.warn('some %s %s', 'warn', 2);

            expect(errorTrackerNotifyStub.callCount).to.equal(1);
            expect(errorTrackerNotifyStub.getCall(0).args).to.deep.equal(['some warn 2', expectedOptions]);
        });
    });

    describe('error', function() {
        var errorTrackerNotifyStub;
        var expectedOptions;

        beforeEach(function() {
            consoleStub = sinon.stub(console, 'error').returns();
            errorTrackerNotifyStub = sinon.stub(errorTracker, 'notify').returns();

            expectedOptions = {
                severity: 'error'
            };
        });

        afterEach(function() {
            console.error.restore();
            errorTracker.notify.restore();
        });

        it('should not do anything if no arguments were passed', function() {
            logger.error();

            expect(consoleStub.callCount).to.equal(0);
            expect(errorTrackerNotifyStub.callCount).to.equal(0);
        });

        it('should simply pass arguments to console.error', function() {
            var error = new Error('some exception');
            logger.error('some %s %s %s', 'error', 2, error);

            expect(consoleStub.callCount).to.equal(1);
            expect(consoleStub.getCall(0).args).to.deep.equal(['some %s %s %s', 'error', 2, error]);
        });

        it('should use utils.format to format the message when sending to Bugsnag', function() {
            logger.error('some %s %s', 'error', 2);

            expect(errorTrackerNotifyStub.callCount).to.equal(1);
            expect(errorTrackerNotifyStub.getCall(0).args).to.deep.equal(['some error 2', expectedOptions]);
        });
    });

    describe('errorStack', function() {
        var errorTrackerNotifyStub;
        var expectedOptions;

        beforeEach(function() {
            consoleStub = sinon.stub(console, 'error').returns();
            errorTrackerNotifyStub = sinon.stub(errorTracker, 'notify').returns();

            expectedOptions = {
                severity: 'error'
            };
        });

        afterEach(function() {
            console.error.restore();
            errorTracker.notify.restore();
        });

        it('should not do anything if no arguments were passed', function() {
            logger.errorStack();

            expect(consoleStub.callCount).to.equal(0);
            expect(errorTrackerNotifyStub.callCount).to.equal(0);
        });

        it('should write error as stack to console.error if available', function() {
            var error = new Error('some exception');
            logger.errorStack(error);

            expect(consoleStub.callCount).to.equal(1);
            expect(consoleStub.getCall(0).args).to.deep.equal([error.stack]);

            expect(errorTrackerNotifyStub.callCount).to.equal(1);
            expect(errorTrackerNotifyStub.getCall(0).args).to.deep.equal([error, expectedOptions]);
        });

        it('should write error to console.error if error is a string', function() {
            logger.errorStack('some-error');

            expect(consoleStub.callCount).to.equal(1);
            expect(consoleStub.getCall(0).args).to.deep.equal(['some-error']);

            expect(errorTrackerNotifyStub.callCount).to.equal(1);
            expect(errorTrackerNotifyStub.getCall(0).args).to.deep.equal(['some-error', expectedOptions]);
        });
    });

    describe('wrap', function() {
        it('should wrap add fields id and errmsg to the passed error', function() {
            var error = new Error('some error');
            expect(error).to.not.have.property('id');
            expect(error).to.not.have.property('errmsg');

            var result = logger.wrap(error);
            expect(result).to.equal(error);
            expect(error).to.have.property('id');
            expect(error.id).to.match(/^ERR\d+$/g);
            expect(error).to.have.property('errmsg', error.message);
        });
    });

    describe('expressLoggerMiddleware', function() {
        it('should get express handler', function() {
            var middleware = logger.expressLoggerMiddleware();
            expect(middleware).to.be.an('function');
            expect(middleware.length).to.be.equal(3);
        });

        describe('logger', function() {
            var memStream;
            var app;
            var request = require('supertest');
            var data;
            beforeEach(function() {
                data =  '';
                memStream = new MemoryStream();
                memStream.on('data', function(chunk) {
                    data += chunk.toString();
                });
                app =  express();
                var middleware = logger.expressLoggerMiddleware({stream: memStream});
                app.use(middleware);
            });

            it('should write in stream', function(done) {
                app.get('/', function(req, res) {
                  res.send('hello, world!');
                });

                request(app)
                  .get('/')
                  .end(function(err) {
                        expect(err).to.be.null;
                        expect(data).to.be.contains('login=-');
                        done();
                  });
            });

            it('should write in stream with logged info', function(done) {
                app.use(function(req, res, next) {
                    req.user = {
                        login: 'lcalvy'
                    };
                    next();
                });

                app.get('/', function(req, res) {
                  res.send('hello, world!');
                });

                request(app)
                  .get('/')
                  .end(function(err) {
                        expect(err).to.be.null;
                        expect(data).to.be.contains('login=lcalvy');
                        done();
                  });
            });

            afterEach(function() {
                memStream.end();
            });
        });
    });
});
